Module: Viewpoint
- Included in:
- EWS::EWS, EWS::GenericFolder
- Defined in:
- lib/viewpoint.rb,
lib/model/item.rb,
lib/model/task.rb,
lib/model/event.rb,
lib/model/model.rb,
lib/model/folder.rb,
lib/model/contact.rb,
lib/model/message.rb,
lib/model/attendee.rb,
lib/model/attachment.rb,
lib/model/mailbox_user.rb,
lib/model/tasks_folder.rb,
lib/soap/soap_provider.rb,
lib/model/calendar_item.rb,
lib/model/search_folder.rb,
lib/model/generic_folder.rb,
lib/soap/handsoap/parser.rb,
lib/exceptions/exceptions.rb,
lib/model/calendar_folder.rb,
lib/model/contacts_folder.rb,
lib/model/file_attachment.rb,
lib/model/item_attachment.rb,
lib/model/meeting_message.rb,
lib/model/meeting_request.rb,
lib/model/meeting_response.rb,
lib/model/distribution_list.rb,
lib/model/item_field_uri_map.rb,
lib/soap/handsoap/ews_service.rb,
lib/model/meeting_cancellation.rb,
lib/soap/handsoap/parsers/ews_parser.rb,
lib/soap/handsoap/builders/ews_builder.rb,
lib/soap/handsoap/builders/ews_build_helpers.rb
Overview
Copyright © 2010 Dan Wanek <[email protected]>
This file is part of Viewpoint.
Viewpoint is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
Viewpoint is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with Viewpoint. If not, see <www.gnu.org/licenses/>.
This is a module that is included in the main Builder that has sub builders that are used from multiple root nodes. It is basically just a way to do code reuse in a more natural way. The one difference between these functions and the builder functions in the EwsBuilder class is that the first parameter is of node type. We use that parameter to build subnodes in this module.
Defined Under Namespace
Modules: EWS